import cfnlint

from hyperscale.ozone.cloudtrail import OrganizationalCloudTrail


def test_organizational_cloudtrail():
    ct = OrganizationalCloudTrail()
    t = ct.create_template()
    errors = cfnlint.lint(
        t.to_json(),
    )
    print(t.to_json())
    assert not errors
    d = t.to_dict()
    params = d["Parameters"]
    assert "LogArchiveBucketName" in params

    resources = d["Resources"]
    assert "OrgTrail" in resources



from troposphere import cloudtrail
from troposphere import Parameter
from troposphere import Ref
from troposphere import Template


class OrganizationalCloudTrail:
    def create_template(self):
        t = Template()
        t.set_description("Organizational CloudTrail Configuration")
        self.add_resources(t)
        return t

    def add_resources(self, t: Template):
        log_archive_bucket_name_param = t.add_parameter(
            Parameter(
                "LogArchiveBucketName",
                Type="String",
                Description="The S3 bucket to store CloudTrail logs",
            )
        )
        t.add_resource(
            cloudtrail.Trail(
                "OrgTrail",
                IsOrganizationTrail=True,
                S3BucketName=Ref(log_archive_bucket_name_param),
                IsMultiRegionTrail=True,
                EnableLogFileValidation=True,
                IncludeGlobalServiceEvents=True,
                IsLogging=True,
            )
        )



import cfnlint

from hyperscale.ozone.route53resolver import Route53ResolverQueryLoggingConfig


def test_organizational_cloudtrail():
    c = Route53ResolverQueryLoggingConfig()
    t = c.create_template()
    errors = cfnlint.lint(
        t.to_json(),
    )
    print(t.to_json())
    assert not errors
    d = t.to_dict()
    params = d["Parameters"]
    assert "Namespace" in params
    assert "LogDestinationArn" in params

    resources = d["Resources"]
    assert "QueryLoggingConfig" in resources



import click

from hyperscale.ozone.cloudtrail import OrganizationalCloudTrail
from hyperscale.ozone.iam import GitHubOIDCProvider
from hyperscale.ozone.iam import GitLabOIDCProvider
from hyperscale.ozone.pipelines import LandingZoneConfigurationPipeline
from hyperscale.ozone.route53resolver import Route53ResolverQueryLoggingConfig
from hyperscale.ozone.rvm import RoleVendingMachine
from hyperscale.ozone.rvm import WorkflowRole
from hyperscale.ozone.s3 import CentralLogArchiveBuckets
from hyperscale.ozone.s3 import CentralS3AccessLogsReplicationRole
from hyperscale.ozone.s3 import LocalAccessLogsBucket
from hyperscale.ozone.s3 import OrganizationAssetsBucket


@click.group()
@click.version_option(package_name="hyperscale.ozone")
def main():
    """Ozone - higher level cloud constructs build on troposphere."""
    pass


TEMPLATES = {
    "organizational-cloudtrail": (
        OrganizationalCloudTrail(),
        "An organizational CloudTrail",
    ),
    "route53-resolver-query-logging-config": (
        Route53ResolverQueryLoggingConfig(),
        "A Route 53 Resolver Query Logging Configuration",
    ),
    "central-log-archive-buckets": (
        CentralLogArchiveBuckets(),
        "Central log archive buckets",
    ),
    "role-vending-machine": (RoleVendingMachine(), "A Role Vending Machine"),
    "rvm-workflow-role": (
        WorkflowRole(),
        "The workflow role for a Role Vending Machine",
    ),
    "local-access-logs-bucket": (
        LocalAccessLogsBucket(),
        "An account-local s3 access logs bucket that replicates logs to a central "
        "account",
    ),
    "github-oidc-provider": (GitHubOIDCProvider(), "An IAM OIDC provider for GitHub"),
    "gitlab-oidc-provider": (GitLabOIDCProvider(), "An IAM OIDC provider for GitLab"),
    "central-s3-access-logs-replication-role": (
        CentralS3AccessLogsReplicationRole(),
        "A role for replicating s3 objects to the central s3 access logs bucket",
    ),
    "landing-zone-configuration-pipeline": (
        LandingZoneConfigurationPipeline(),
        "A pipeline for landing zone configuration",
    ),
    "organization-assets-bucket": (
        OrganizationAssetsBucket(),
        "An organization assets bucket",
    ),
}


@main.command()
@click.argument("template", type=click.Choice(TEMPLATES.keys()), required=True)
def create_template(template):
    """Generate the specified CloudFormation template"""
    if template not in TEMPLATES:
        raise click.ClickException(f"Unknown template: {template}")
    factory, _ = TEMPLATES[template]
    click.echo(factory.create_template().to_yaml())


@main.command()
def list_templates():
    """List all available templates"""
    for k, v in TEMPLATES.items():
        click.echo(f"{k}: {v[1]}")



import cfnlint

from hyperscale.ozone.pipelines import LandingZoneConfigurationPipeline


def test_lz_pipeline():
    pipeline = LandingZoneConfigurationPipeline()
    t = pipeline.create_template()
    errors = cfnlint.lint(
        t.to_json(),
    )
    assert not errors



from dataclasses import dataclass

from troposphere import codebuild
from troposphere import codepipeline
from troposphere import GetAtt
from troposphere import iam
from troposphere import Output
from troposphere import Parameter
from troposphere import Ref
from troposphere import s3
from troposphere import Sub
from troposphere import Template

from hyperscale.ozone.iam import GitHubOIDCProvider
from hyperscale.ozone.s3 import SecureS3


class LandingZoneConfigurationPipeline:
    def create_template(self):
        template = Template()
        template.set_description(
            "A pipeline that maintains a landing zone using signed config files "
            "deployed from github"
        )
        self.add_resources(template)
        return template

    def add_resources(self, template: Template):
        oidc_provider = GitHubOIDCProvider()
        oidc_provider.add_resources(template)

        access_logs_bucket = SecureS3(
            "AccessLogs",
            bucket_name=Sub("${Namespace}-config-access-logs-${AWS::AccountId}"),
            is_access_logs_bucket=True,
            policy_statements=[
                {
                    "Effect": "Allow",
                    "Principal": {"Service": "logging.s3.amazonaws.com"},
                    "Action": "s3:PutObject",
                    "Resource": Sub("${AccessLogsBucket.Arn}/*"),
                    "Condition": {
                        "StringEquals": {"aws:SourceAccount": Sub("${AWS::AccountId}")},
                    },
                },
            ],
            retention_days=365,
        )
        access_logs_bucket.add_resources(template)

        pipeline = StaxPipeline(
            oidc_provider=oidc_provider.oidc_provider,
            s3_access_logs_bucket=access_logs_bucket.bucket,
        )
        pipeline.add_resources(template)


@dataclass
class StaxPipeline:
    oidc_provider: iam.OIDCProvider | Parameter | None = None
    s3_access_logs_bucket: s3.Bucket | Parameter | None = None
    admin_role_arn: iam.Role | None = None
    execution_role_name: str | None = None

    def create_template(self):
        template = Template()
        template.set_description(
            "A pipeline that deploys stack sets using stax and signed config files "
            "deployed from github"
        )
        self.add_resources(template)
        return template

    def add_resources(self, template: Template):
        template.add_parameter(
            Parameter(
                "Namespace",
                Description="The namespace for stacks deployed by this pipeline",
                Type="String",
            )
        )
        template.add_parameter(
            Parameter(
                "PublishWorkflow",
                Description="The trusted publisher workflow e.g. publish.yaml",
                Type="String",
            )
        )
        template.add_parameter(
            Parameter(
                "RepoOwner",
                Description="The owner of the trusted publishing github repo",
                Type="String",
            )
        )
        template.add_parameter(
            Parameter(
                "Repo",
                Description="The trusted publishing repo",
                Type="String",
            )
        )

        if not self.oidc_provider:
            self.oidc_provider = template.add_parameter(
                Parameter(
                    "GitHubOIDCProviderArn",
                    Type="String",
                    Description="The GitHub OIDC Provider ARN",
                )
            )

        publishing_role = template.add_resource(
            iam.Role(
                "PublishingRole",
                AssumeRolePolicyDocument={
                    "Version": "2012-10-17",
                    "Statement": [
                        {
                            "Effect": "Allow",
                            "Principal": {"Federated": Ref(self.oidc_provider)},
                            "Action": "sts:AssumeRoleWithWebIdentity",
                            "Condition": {
                                "StringEquals": {
                                    "token.actions.githubusercontent.com:aud": (
                                        "sts.amazonaws.com"
                                    ),
                                },
                                "StringLike": {
                                    "token.actions.githubusercontent.com:sub": Sub(
                                        "repo:${RepoOwner}/${Repo}:ref:refs/tags/v*"
                                    )
                                },
                            },
                        }
                    ],
                },
                Policies=[
                    iam.Policy(
                        PolicyName="PublishLzArchive",
                        PolicyDocument={
                            "Version": "2012-10-17",
                            "Statement": [
                                {
                                    "Action": "s3:PutObject",
                                    "Resource": Sub(
                                        "arn:${AWS::Partition}:s3:::${SourceBucket}/*"
                                    ),
                                    "Effect": "Allow",
                                }
                            ],
                        },
                    )
                ],
            )
        )

        if not self.s3_access_logs_bucket:
            self.s3_access_logs_bucket = template.add_parameter(
                Parameter(
                    "S3AccessLogsBucket",
                    Type="String",
                    Description="The bucket for S3 access logs",
                )
            )
        source_bucket = SecureS3(
            "Source",
            bucket_name=Sub("${Namespace}-config-source-${AWS::AccountId}"),
            access_logs_bucket=Ref(self.s3_access_logs_bucket),
            retention_days=7,
        )
        source_bucket.add_resources(template)

        artifact_bucket = SecureS3(
            "Artifact",
            bucket_name=Sub("${Namespace}-config-artifact-${AWS::AccountId}"),
            access_logs_bucket=Ref(self.s3_access_logs_bucket),
            retention_days=7,
        )
        artifact_bucket.add_resources(template)

        pipeline_role = template.add_resource(
            iam.Role(
                "PipelineRole",
                AssumeRolePolicyDocument={
                    "Version": "2012-10-17",
                    "Statement": [
                        {
                            "Effect": "Allow",
                            "Principal": {"Service": "codepipeline.amazonaws.com"},
                            "Action": "sts:AssumeRole",
                        }
                    ],
                },
                Policies=[],
            )
        )

        source_action_role = template.add_resource(
            iam.Role(
                "SourceActionRole",
                AssumeRolePolicyDocument={
                    "Version": "2012-10-17",
                    "Statement": [
                        {
                            "Effect": "Allow",
                            "Principal": {"AWS": GetAtt(pipeline_role, "Arn")},
                            "Action": "sts:AssumeRole",
                        }
                    ],
                },
                Policies=[
                    iam.Policy(
                        PolicyName="PipelinePolicy",
                        PolicyDocument={
                            "Version": "2012-10-17",
                            "Statement": [
                                {
                                    "Effect": "Allow",
                                    "Action": [
                                        "s3:ListBucket",
                                        "s3:GetObject",
                                        "s3:GetObjectVersion",
                                        "s3:GetBucketVersioning",
                                        "s3:GetBucketAcl",
                                        "s3:GetBucketLocation",
                                        "s3:GetObjectTagging",
                                        "s3:GetObjectVersionTagging",
                                    ],
                                    "Resource": [
                                        Sub(
                                            "arn:${AWS::Partition}:s3:::${SourceBucket}"
                                        ),
                                        Sub(
                                            "arn:${AWS::Partition}:s3:::${SourceBucket}/*"
                                        ),
                                    ],
                                },
                                {
                                    "Effect": "Allow",
                                    "Action": [
                                        "s3:ListBucket",
                                    ],
                                    "Resource": Sub(
                                        "arn:${AWS::Partition}:s3:::${ArtifactBucket}"
                                    ),
                                },
                                {
                                    "Effect": "Allow",
                                    "Action": [
                                        "s3:PutObject",
                                    ],
                                    "Resource": Sub(
                                        "arn:${AWS::Partition}:s3:::${ArtifactBucket}/*"
                                    ),
                                },
                            ],
                        },
                    ),
                ],
            )
        )

        build_action_role = template.add_resource(
            iam.Role(
                "BuildActionRole",
                AssumeRolePolicyDocument={
                    "Version": "2012-10-17",
                    "Statement": [
                        {
                            "Effect": "Allow",
                            "Principal": {"AWS": GetAtt(pipeline_role, "Arn")},
                            "Action": "sts:AssumeRole",
                        }
                    ],
                },
                Policies=[
                    iam.Policy(
                        PolicyName="BuildActionPolicy",
                        PolicyDocument={
                            "Version": "2012-10-17",
                            "Statement": [
                                {
                                    "Effect": "Allow",
                                    "Action": [
                                        "codebuild:BatchGetBuilds",
                                        "codebuild:StartBuild",
                                        "codebuild:BatchGetBuildBatches",
                                        "codebuild:StartBuildBatch",
                                    ],
                                    "Resource": Sub(
                                        "arn:${AWS::Partition}:codebuild:${AWS::Region}:${AWS::AccountId}:project/${BuildProject}"
                                    ),
                                }
                            ],
                        },
                    ),
                ],
            )
        )

        codebuild_role = template.add_resource(
            iam.Role(
                "CodebuildRole",
                AssumeRolePolicyDocument={
                    "Version": "2012-10-17",
                    "Statement": [
                        {
                            "Effect": "Allow",
                            "Principal": {"Service": "codebuild.amazonaws.com"},
                            "Action": "sts:AssumeRole",
                        }
                    ],
                },
                Policies=[
                    iam.Policy(
                        PolicyName="CodebuildPolicy",
                        PolicyDocument={
                            "Version": "2012-10-17",
                            "Statement": [
                                {
                                    "Sid": "ManageStackSets",
                                    "Effect": "Allow",
                                    "Action": [
                                        "cloudformation:CreateStackInstances",
                                        "cloudformation:DeleteStackInstances",
                                        "cloudformation:DeleteStack",
                                        "cloudformation:ListStackInstances",
                                        "cloudformation:DescribeStackSet",
                                        "cloudformation:DescribeStacks",
                                        "cloudformation:UpdateStackSet",
                                        "cloudformation:GetTemplateSummary",
                                        "cloudformation:TagResource",
                                    ],
                                    "Resource": [
                                        Sub(
                                            "arn:${AWS::Partition}:cloudformation:${AWS::Region}:${AWS::AccountId}:stackset/${Namespace}-*",
                                        ),
                                        Sub(
                                            "arn:${AWS::Partition}:cloudformation:${AWS::Region}:${AWS::AccountId}:stack/${Namespace}-*",
                                        ),
                                        Sub(
                                            "arn:${AWS::Partition}:cloudformation:*:${AWS::AccountId}:stackset-target/${Namespace}-*",
                                        ),
                                        Sub(
                                            "arn:${AWS::Partition}:cloudformation:${AWS::Region}::type/resource/*",
                                        ),
                                    ],
                                },
                                {
                                    "Sid": "ManageStackSetsOnNonSpecificResources",
                                    "Effect": "Allow",
                                    "Action": [
                                        "cloudformation:CreateStackSet",
                                        "cloudformation:CreateStack",
                                        "cloudformation:DescribeStackSet",
                                        "cloudformation:ListStackSets",
                                        "cloudformation:ListStacks",
                                    ],
                                    "Resource": "*",
                                },
                                {
                                    "Sid": "S3Access",
                                    "Effect": "Allow",
                                    "Action": "s3:GetObject",
                                    "Resource": Sub(
                                        "arn:${AWS::Partition}:s3:::${ArtifactBucket}/*"
                                    ),
                                },
                                {
                                    "Sid": "CreateLogs",
                                    "Effect": "Allow",
                                    "Action": [
                                        "logs:CreateLogGroup",
                                        "logs:CreateLogStream",
                                        "logs:PutLogEvents",
                                    ],
                                    "Resource": [
                                        Sub(
                                            "arn:${AWS::Partition}:logs:${AWS::Region}:${AWS::AccountId}:log-group:/aws/codebuild/*"
                                        ),
                                        Sub(
                                            "arn:${AWS::Partition}:logs:${AWS::Region}:${AWS::AccountId}:log-group:/aws/codebuild/*:*",
                                        ),
                                    ],
                                },
                                {
                                    "Sid": "PassRole",
                                    "Effect": "Allow",
                                    "Action": "iam:PassRole",
                                    "Resource": "*",
                                },
                            ],
                        },
                    )
                ],
            )
        )

        build_project = template.add_resource(
            codebuild.Project(
                "BuildProject",
                Artifacts=codebuild.Artifacts(
                    Type="CODEPIPELINE",
                ),
                Environment=codebuild.Environment(
                    ComputeType="BUILD_GENERAL1_SMALL",
                    Image="ghcr.io/hyperscale-consulting/stax@sha256:cb58976dc7479dced3afc4c803f56faa6035338dfc9720354794d29b490cc401",
                    Type="LINUX_CONTAINER",
                ),
                Source=codebuild.Source(
                    Type="CODEPIPELINE",
                    BuildSpec=Sub(
                        _build_spec(self.admin_role_arn, self.execution_role_name)
                    ),
                ),
                ServiceRole=GetAtt(codebuild_role, "Arn"),
            )
        )

        template.add_resource(
            codepipeline.Pipeline(
                "Pipeline",
                ArtifactStore=codepipeline.ArtifactStore(
                    Type="S3",
                    Location=Ref(artifact_bucket.bucket),
                ),
                PipelineType="V1",
                RoleArn=GetAtt(pipeline_role, "Arn"),
                Stages=[
                    codepipeline.Stages(
                        Name="Source",
                        Actions=[
                            codepipeline.Actions(
                                Name="Source",
                                ActionTypeId=codepipeline.ActionTypeId(
                                    Category="Source",
                                    Owner="AWS",
                                    Provider="S3",
                                    Version="1",
                                ),
                                OutputArtifacts=[
                                    codepipeline.OutputArtifacts(Name="Source")
                                ],
                                Configuration={
                                    "S3Bucket": Ref(source_bucket.bucket),
                                    "S3ObjectKey": Sub("${Namespace}-archive.zip"),
                                },
                                RoleArn=GetAtt(source_action_role, "Arn"),
                            )
                        ],
                    ),
                    codepipeline.Stages(
                        Name="Build",
                        Actions=[
                            codepipeline.Actions(
                                Name="Build",
                                ActionTypeId=codepipeline.ActionTypeId(
                                    Category="Build",
                                    Owner="AWS",
                                    Provider="CodeBuild",
                                    Version="1",
                                ),
                                InputArtifacts=[
                                    codepipeline.InputArtifacts(Name="Source")
                                ],
                                Configuration={
                                    "ProjectName": Ref(build_project),
                                    "PrimarySource": "Source",
                                },
                                RoleArn=GetAtt(build_action_role, "Arn"),
                            )
                        ],
                    ),
                ],
            )
        )
        template.add_output(
            Output(
                "PublishingRole",
                Description="The role assumed by the trusted publisher",
                Value=Ref(publishing_role),
            )
        )


def _build_spec(admin_role_arn: iam.Role | None, execution_role_name: str | None):
    self_managed_args = (
        admin_role_arn
        and f"-a ${{{admin_role_arn.title}.Arn}} -e {execution_role_name}"
        or ""
    )
    return (
        "version: 0.2\n"
        "phases:\n"
        "  build:\n"
        "    commands:\n"
        "      - ls\n"
        "      - stax deploy -n ${Namespace} "
        f"{self_managed_args} "
        "-s ${Namespace}-config.zip.sigstore.json "
        "-i https://github.com/${RepoOwner}/${Repo}/.github/workflows/"
        "${PublishWorkflow}@refs/tags/v$(cat VERSION.txt) "
        "-r https://token.actions.githubusercontent.com "
        "${Namespace}-config.zip\n"
    )



from troposphere import Output
from troposphere import Parameter
from troposphere import Ref
from troposphere import route53resolver
from troposphere import ssm
from troposphere import Sub
from troposphere import Template


class Route53ResolverQueryLoggingConfig:
    def create_template(self):
        t = Template()
        t.set_description("Route 53 Resolver Query Logging Configuration")
        self.add_resources(t)
        return t

    def add_resources(self, t: Template):
        t.add_parameter(
            Parameter(
                "Namespace",
                Type="String",
                Description=(
                    "The namespace for the SSM parameter containing the config ID"
                ),
                Default="/hyperscale/ozone",
            )
        )
        log_destination_arn_param = t.add_parameter(
            Parameter(
                "LogDestinationArn",
                Type="String",
                Description="The ARN of the destination to send query logs to",
            )
        )
        config = t.add_resource(
            route53resolver.ResolverQueryLoggingConfig(
                "QueryLoggingConfig",
                DestinationArn=Ref(log_destination_arn_param),
            )
        )
        t.add_resource(
            ssm.Parameter(
                "Route53ResolverQueryLoggingConfigId",
                Name=Sub("${Namespace}/route53resolver/query-logging-config/id"),
                Type="String",
                Value=Ref(config),
                Description=(
                    "The ID of the Route 53 Resolver Query Logging Configuration"
                ),
            )
        )
        t.add_output(Output("ResolverQueryLoggingConfigId", Value=Ref(config)))



from troposphere import iam
from troposphere import Parameter
from troposphere import Ref
from troposphere import Sub
from troposphere import Template

from hyperscale.ozone import cfn_nag
from hyperscale.ozone.pipelines import StaxPipeline


class RoleVendingMachine:
    def create_template(self) -> Template:
        template = Template()
        template.set_description("Role Vending Machine")
        self.add_resources(template)
        return template

    def add_resources(self, template: Template) -> None:
        rvm_main_role = template.add_resource(
            iam.Role(
                "RvmMainRole",
                Metadata=cfn_nag.suppress(
                    [
                        cfn_nag.rule(
                            id="W28",
                            reason="Static role name so it can be easily referred to "
                            "in the RVM workflow role policy",
                        )
                    ]
                ),
                RoleName="RvmMainRole",
                AssumeRolePolicyDocument={
                    "Version": "2012-10-17",
                    "Statement": [
                        {
                            "Effect": "Allow",
                            "Action": "sts:AssumeRole",
                            "Principal": {"Service": "cloudformation.amazonaws.com"},
                            "Condition": {
                                "StringEquals": {
                                    "aws:SourceAccount": Ref("AWS::AccountId")
                                },
                                "ArnLike": {
                                    "aws:SourceArn": Sub(
                                        "arn:${AWS::Partition}:cloudformation:*:${AWS::AccountId}:stackset/pch-role-vending-machine-*"
                                    )
                                },
                            },
                        }
                    ],
                },
                Policies=[
                    iam.Policy(
                        PolicyName="RVMPolicy",
                        PolicyDocument={
                            "Version": "2012-10-17",
                            "Statement": [
                                {
                                    "Effect": "Allow",
                                    "Action": "sts:AssumeRole",
                                    "Resource": Sub(
                                        "arn:${AWS::Partition}:iam::*:role/RvmWorkflowRole"
                                    ),
                                }
                            ],
                        },
                    )
                ],
            )
        )
        pipelines = StaxPipeline(
            admin_role_arn=rvm_main_role,
            execution_role_name="RvmWorkflowRole",
        )
        pipelines.add_resources(template)


class WorkflowRole:
    """
    Role Vending Machine workflow roles that get deployed to each RVM managed account.
    """

    def create_template(self) -> Template:
        template = Template()
        template.set_description(
            "Role Vending Machine workflow roles that get deployed to each RVM "
            "managed account."
        )
        self.add_resources(template)
        return template

    def add_resources(self, template: Template) -> None:
        template.add_parameter(
            Parameter(
                "RvmAccount",
                Type="String",
                Description="The ID of the RVM account",
            )
        )
        template.add_resource(
            iam.Role(
                "RvmWorkflowRole",
                Metadata=cfn_nag.suppress(
                    [
                        cfn_nag.rule("W11", "Need to be able to manage all roles"),
                        cfn_nag.rule(
                            "W28",
                            "Static role name so it can be easily referred to in "
                            "the RVM main role policy",
                        ),
                    ]
                ),
                RoleName="RvmWorkflowRole",
                Description="The role assumed by the RVM Main Role from the RVM "
                "account to vend new roles",
                AssumeRolePolicyDocument={
                    "Version": "2012-10-17",
                    "Statement": [
                        {
                            "Effect": "Allow",
                            "Principal": {
                                "AWS": Sub(
                                    "arn:${AWS::Partition}:iam::${RvmAccount}:role/RvmMainRole"
                                )
                            },
                            "Action": "sts:AssumeRole",
                        }
                    ],
                },
                Policies=[
                    iam.Policy(
                        PolicyName="AllowManagePermissions",
                        PolicyDocument={
                            "Version": "2012-10-17",
                            "Statement": [
                                {
                                    "Effect": "Allow",
                                    "Action": [
                                        "iam:AttachRolePolicy",
                                        "iam:CreateRole",
                                        "iam:CreatePolicy",
                                        "iam:CreatePolicyVersion",
                                        "iam:DeleteRolePolicy",
                                        "iam:DeleteRole",
                                        "iam:DeletePolicy",
                                        "iam:DetachRolePolicy",
                                        "iam:GetPolicy",
                                        "iam:GetPolicyVersion",
                                        "iam:GetRole",
                                        "iam:GetRolePolicy",
                                        "iam:ListAttachedRolePolicies",
                                        "iam:ListRoles",
                                        "iam:ListPolicies",
                                        "iam:UpdateRole",
                                        "iam:PutRolePolicy",
                                        "iam:SetDefaultPolicyVersion",
                                        "iam:TagRole",
                                    ],
                                    "Resource": "*",
                                }
                            ],
                        },
                    ),
                    iam.Policy(
                        PolicyName="AllowManageRvmStacks",
                        PolicyDocument={
                            "Version": "2012-10-17",
                            "Statement": [
                                {
                                    "Effect": "Allow",
                                    "Action": [
                                        "cloudformation:*",
                                    ],
                                    "Resource": "*",
                                },
                            ],
                        },
                    ),
                ],
            )
        )
